@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -16,14 +16,14 @@
 #  You should have received a copy of the GNU Lesser General Public License
 #  along with telectron.  If not, see <http://www.gnu.org/licenses/>.
 
-__version__ = "1.2.38"
-__license__ = "GNU Lesser General Public License v3 or later (LGPLv3+)"
-__copyright__ = "Copyright (C) 2017-2021 Dan <https://github.com/delivrance>"
+__version__ = "2.0.113.1"
+__license__ = "GNU Lesser General Public License v3.0 (LGPL-3.0)"
+__copyright__ = "Copyright (C) 2017-present Dan <https://github.com/delivrance>"
 
 from concurrent.futures.thread import ThreadPoolExecutor
 
 
-class StopTransmission(StopAsyncIteration):
+class StopTransmission(Exception):
     pass
 
 
@@ -35,15 +35,8 @@ class ContinuePropagation(StopAsyncIteration):
     pass
 
 
-import asyncio
-
-from . import raw, types, filters, handlers, emoji
+from . import raw, types, filters, handlers, emoji, enums
 from .client import Client
-from .sync import idle
-
-# Save the main thread loop for future references
-main_event_loop = asyncio.get_event_loop()
-
-CRYPTO_EXECUTOR_SIZE_THRESHOLD = 512
+from .sync import idle, compose
 
 crypto_executor = ThreadPoolExecutor(1, thread_name_prefix="CryptoWorker")
