@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -24,9 +24,9 @@ from collections import OrderedDict
 import telectron
 from telectron import utils
 from telectron.handlers import (
-    CallbackQueryHandler, MessageHandler, DeletedMessagesHandler,
+    CallbackQueryHandler, MessageHandler, EditedMessageHandler, DeletedMessagesHandler,
     UserStatusHandler, RawUpdateHandler, InlineQueryHandler, PollHandler,
-    ChosenInlineResultHandler, ChatMemberUpdatedHandler
+    ChosenInlineResultHandler, ChatMemberUpdatedHandler, ChatJoinRequestHandler
 )
 from telectron.raw.types import (
     UpdateNewMessage, UpdateNewChannelMessage, UpdateNewScheduledMessage,
@@ -34,42 +34,27 @@ from telectron.raw.types import (
     UpdateDeleteMessages, UpdateDeleteChannelMessages,
     UpdateBotCallbackQuery, UpdateInlineBotCallbackQuery,
     UpdateUserStatus, UpdateBotInlineQuery, UpdateMessagePoll,
-    UpdateBotInlineSend, UpdateChatParticipant, UpdateChannelParticipant
+    UpdateBotInlineSend, UpdateChatParticipant, UpdateChannelParticipant,
+    UpdateBotChatInviteRequester
 )
 
 log = logging.getLogger(__name__)
 
 
 class Dispatcher:
-    NEW_MESSAGE_UPDATES = (
-        UpdateNewMessage,
-        UpdateNewChannelMessage,
-        UpdateNewScheduledMessage
-    )
-
-    EDIT_MESSAGE_UPDATES = (
-        UpdateEditMessage,
-        UpdateEditChannelMessage,
-    )
-
-    DELETE_MESSAGES_UPDATES = (
-        UpdateDeleteMessages,
-        UpdateDeleteChannelMessages
-    )
-
-    CALLBACK_QUERY_UPDATES = (
-        UpdateBotCallbackQuery,
-        UpdateInlineBotCallbackQuery
-    )
-
-    CHAT_MEMBER_UPDATES = (
-        UpdateChatParticipant,
-        UpdateChannelParticipant
-    )
-
-    MESSAGE_UPDATES = NEW_MESSAGE_UPDATES + EDIT_MESSAGE_UPDATES
+    NEW_MESSAGE_UPDATES = (UpdateNewMessage, UpdateNewChannelMessage, UpdateNewScheduledMessage)
+    EDIT_MESSAGE_UPDATES = (UpdateEditMessage, UpdateEditChannelMessage)
+    DELETE_MESSAGES_UPDATES = (UpdateDeleteMessages, UpdateDeleteChannelMessages)
+    CALLBACK_QUERY_UPDATES = (UpdateBotCallbackQuery, UpdateInlineBotCallbackQuery)
+    CHAT_MEMBER_UPDATES = (UpdateChatParticipant, UpdateChannelParticipant)
+    USER_STATUS_UPDATES = (UpdateUserStatus,)
+    BOT_INLINE_QUERY_UPDATES = (UpdateBotInlineQuery,)
+    POLL_UPDATES = (UpdateMessagePoll,)
+    CHOSEN_INLINE_RESULT_UPDATES = (UpdateBotInlineSend,)
+    CHAT_JOIN_REQUEST_UPDATES = (UpdateBotChatInviteRequester,)
 
     def __init__(self, client: "telectron.Client"):
+        self.waiting_events = {}
         self.client = client
         self.loop = asyncio.get_event_loop()
 
@@ -78,44 +63,82 @@ class Dispatcher:
 
         self.updates_queue = asyncio.Queue()
         self.groups = OrderedDict()
-        self.waiting_events = {}
 
         async def message_parser(update, users, chats):
-            return await telectron.types.Message._parse(
-                self.client, update.message, users, chats,
-                isinstance(update, UpdateNewScheduledMessage)
-            ), MessageHandler
+            return (
+                await telectron.types.Message._parse(self.client, update.message, users, chats,
+                                                    isinstance(update, UpdateNewScheduledMessage)),
+                MessageHandler
+            )
+
+        async def edited_message_parser(update, users, chats):
+            # Edited messages are parsed the same way as new messages, but the handler is different
+            parsed, _ = await message_parser(update, users, chats)
+
+            return (
+                parsed,
+                EditedMessageHandler
+            )
 
         async def deleted_messages_parser(update, users, chats):
-            return utils.parse_deleted_messages(self.client, update), DeletedMessagesHandler
+            return (
+                utils.parse_deleted_messages(self.client, update),
+                DeletedMessagesHandler
+            )
 
         async def callback_query_parser(update, users, chats):
-            return await telectron.types.CallbackQuery._parse(self.client, update, users), CallbackQueryHandler
+            return (
+                await telectron.types.CallbackQuery._parse(self.client, update, users),
+                CallbackQueryHandler
+            )
 
         async def user_status_parser(update, users, chats):
-            return telectron.types.User._parse_user_status(self.client, update), UserStatusHandler
+            return (
+                telectron.types.User._parse_user_status(self.client, update),
+                UserStatusHandler
+            )
 
         async def inline_query_parser(update, users, chats):
-            return telectron.types.InlineQuery._parse(self.client, update, users), InlineQueryHandler
+            return (
+                telectron.types.InlineQuery._parse(self.client, update, users),
+                InlineQueryHandler
+            )
 
         async def poll_parser(update, users, chats):
-            return telectron.types.Poll._parse_update(self.client, update), PollHandler
+            return (
+                telectron.types.Poll._parse_update(self.client, update),
+                PollHandler
+            )
 
         async def chosen_inline_result_parser(update, users, chats):
-            return telectron.types.ChosenInlineResult._parse(self.client, update, users), ChosenInlineResultHandler
+            return (
+                telectron.types.ChosenInlineResult._parse(self.client, update, users),
+                ChosenInlineResultHandler
+            )
 
         async def chat_member_updated_parser(update, users, chats):
-            return telectron.types.ChatMemberUpdated._parse(self.client, update, users, chats), ChatMemberUpdatedHandler
+            return (
+                telectron.types.ChatMemberUpdated._parse(self.client, update, users, chats),
+                ChatMemberUpdatedHandler
+            )
+
+        async def chat_join_request_parser(update, users, chats):
+            return (
+                telectron.types.ChatJoinRequest._parse(self.client, update, users, chats),
+                ChatJoinRequestHandler
+            )
 
         self.update_parsers = {
-            Dispatcher.MESSAGE_UPDATES: message_parser,
+            Dispatcher.NEW_MESSAGE_UPDATES: message_parser,
+            Dispatcher.EDIT_MESSAGE_UPDATES: edited_message_parser,
             Dispatcher.DELETE_MESSAGES_UPDATES: deleted_messages_parser,
             Dispatcher.CALLBACK_QUERY_UPDATES: callback_query_parser,
-            (UpdateUserStatus,): user_status_parser,
-            (UpdateBotInlineQuery,): inline_query_parser,
-            (UpdateMessagePoll,): poll_parser,
-            (UpdateBotInlineSend,): chosen_inline_result_parser,
-            Dispatcher.CHAT_MEMBER_UPDATES: chat_member_updated_parser
+            Dispatcher.USER_STATUS_UPDATES: user_status_parser,
+            Dispatcher.BOT_INLINE_QUERY_UPDATES: inline_query_parser,
+            Dispatcher.POLL_UPDATES: poll_parser,
+            Dispatcher.CHOSEN_INLINE_RESULT_UPDATES: chosen_inline_result_parser,
+            Dispatcher.CHAT_MEMBER_UPDATES: chat_member_updated_parser,
+            Dispatcher.CHAT_JOIN_REQUEST_UPDATES: chat_join_request_parser
         }
 
         self.update_parsers = {key: value for key_tuple, value in self.update_parsers.items() for key in key_tuple}
@@ -129,7 +152,7 @@ class Dispatcher:
                     self.loop.create_task(self.handler_worker(self.locks_list[-1]))
                 )
 
-            log.info(f"Started {self.client.workers} HandlerTasks")
+            log.info("Started %s HandlerTasks", self.client.workers)
 
     async def stop(self):
         if not self.client.no_updates:
@@ -142,7 +165,7 @@ class Dispatcher:
             self.handler_worker_tasks.clear()
             self.groups.clear()
 
-            log.info(f"Stopped {self.client.workers} HandlerTasks")
+            log.info("Stopped %s HandlerTasks", self.client.workers)
 
     def add_handler(self, handler, group: int):
         async def fn():
@@ -204,7 +227,7 @@ class Dispatcher:
                                     if await handler.check(self.client, parsed_update):
                                         args = (parsed_update,)
                                 except Exception as e:
-                                    log.error(e, exc_info=True)
+                                    log.exception(e)
                                     continue
 
                             elif isinstance(handler, RawUpdateHandler):
@@ -215,45 +238,35 @@ class Dispatcher:
 
                             try:
                                 if inspect.iscoroutinefunction(handler.callback):
-                                    result = await handler.callback(self.client, *args)
+                                    await handler.callback(self.client, *args)
                                 else:
-                                    result = await self.loop.run_in_executor(
+                                    await self.loop.run_in_executor(
                                         self.client.executor,
                                         handler.callback,
                                         self.client,
                                         *args
                                     )
-                                await handler_type.use_result(self.client, *args, result)
                             except telectron.StopPropagation:
                                 raise
                             except telectron.ContinuePropagation:
                                 continue
                             except Exception as e:
-                                log.error(e, exc_info=True)
+                                log.exception(e)
 
                             break
                     for key, event in self.waiting_events.items():
-                        if event.handler_type == handler_type:
+                        if isinstance(event.handler, handler_type):
                             try:
-                                filters_passed = await event.handler_type.execute_filters(
-                                    event.filters, self.client, parsed_update
-                                )
-                                if filters_passed:
+                                if await event.handler.check(self.client, parsed_update):
                                     async with self.waiting_events[key].lock:
                                         if not self.waiting_events[key].update:
-                                            self.waiting_events[key].update = update
+                                            self.waiting_events[key].update = parsed_update
                                             self.waiting_events[key].event.set()
                             except Exception as e:
                                 log.error(e, exc_info=True)
                                 continue
 
-                        elif event.handler_type == RawUpdateHandler:
-                            async with self.waiting_events[key].lock:
-                                if not self.waiting_events[key].update:
-                                    self.waiting_events[key].update = update
-                                    self.waiting_events[key].event.set()
-
             except telectron.StopPropagation:
                 pass
             except Exception as e:
-                log.error(e, exc_info=True)
+                log.exception(e)
