@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -32,7 +32,7 @@ class DeletedMessagesHandler(Handler):
     :meth:`~telectron.Client.on_deleted_messages` decorator.
 
     Parameters:
-        callback (``callable``):
+        callback (``Callable``):
             Pass a function that will be called when one or more messages have been deleted.
             It takes *(client, messages)* as positional arguments (look at the section below for a detailed description).
 
@@ -51,15 +51,11 @@ class DeletedMessagesHandler(Handler):
     def __init__(self, callback: Callable, filters: Filter = None):
         super().__init__(callback, filters)
 
-    async def check(self, client: "telectron.Client", update: List[Message]):
-        return await self.execute_filters(self.filters, client, update)
-
-    @staticmethod
-    async def execute_filters(filters, client, update: List[Message]):
+    async def check(self, client: "telectron.Client", messages: List[Message]):
         # Every message should be checked, if at least one matches the filter True is returned
         # otherwise, or if the list is empty, False is returned
-        for message in update:
-            if await Handler.execute_filters(filters, client, message):
+        for message in messages:
+            if await super().check(client, message):
                 return True
         else:
-            return False
\ No newline at end of file
+            return False
