@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -30,21 +30,14 @@ class Handler:
         self.filters = filters
 
     async def check(self, client: "telectron.Client", update: Update):
-        return await self.execute_filters(self.filters, client, update)
-
-    @staticmethod
-    async def execute_filters(filters, client, update):
-        if callable(filters):
-            if inspect.iscoroutinefunction(filters.__call__):
-                return await filters(client, update)
+        if callable(self.filters):
+            if inspect.iscoroutinefunction(self.filters.__call__):
+                return await self.filters(client, update)
             else:
                 return await client.loop.run_in_executor(
                     client.executor,
-                    filters,
+                    self.filters,
                     client, update
                 )
-        return True
 
-    @classmethod
-    async def use_result(cls, client, update, result):
-        pass
+        return True
