@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -16,16 +16,22 @@
 #  You should have received a copy of the GNU Lesser General Public License
 #  along with telectron.  If not, see <http://www.gnu.org/licenses/>.
 
+from .invoke import Invoke
 from .resolve_peer import ResolvePeer
 from .save_file import SaveFile
-from .send import Send
 from .wait_for import WaitFor
+from .stream_file import StreamFile
+from .handle_download import HandleDownload
+from .handle_updates import HandleUpdates
 
 
 class Advanced(
+    Invoke,
     ResolvePeer,
     SaveFile,
-    Send,
-    WaitFor
+    StreamFile,
+    WaitFor,
+    HandleDownload,
+    HandleUpdates
 ):
     pass
