@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -16,19 +16,24 @@
 #  You should have received a copy of the GNU Lesser General Public License
 #  along with telectron.  If not, see <http://www.gnu.org/licenses/>.
 
+import telectron
 from telectron import raw
-from telectron.scaffold import Scaffold
 
 
-class AcceptTermsOfService(Scaffold):
-    async def accept_terms_of_service(self, terms_of_service_id: str) -> bool:
+class AcceptTermsOfService:
+    async def accept_terms_of_service(
+        self: "telectron.Client",
+        terms_of_service_id: str
+    ) -> bool:
         """Accept the given terms of service.
 
+        .. include:: /_includes/usable-by/users.rst
+
         Parameters:
             terms_of_service_id (``str``):
                 The terms of service identifier.
         """
-        r = await self.send(
+        r = await self.invoke(
             raw.functions.help.AcceptTermsOfService(
                 id=raw.types.DataJSON(
                     data=terms_of_service_id
@@ -36,6 +41,4 @@ class AcceptTermsOfService(Scaffold):
             )
         )
 
-        assert r
-
-        return True
+        return bool(r)
