@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -18,17 +18,21 @@
 
 import logging
 
+import telectron
 from telectron import raw
-from telectron.scaffold import Scaffold
 
 log = logging.getLogger(__name__)
 
 
-class GetPasswordHint(Scaffold):
-    async def get_password_hint(self) -> str:
+class GetPasswordHint:
+    async def get_password_hint(
+        self: "telectron.Client",
+    ) -> str:
         """Get your Two-Step Verification password hint.
 
+        .. include:: /_includes/usable-by/users.rst
+
         Returns:
             ``str``: On success, the password hint as string is returned.
         """
-        return (await self.send(raw.functions.account.GetPassword())).hint
+        return (await self.invoke(raw.functions.account.GetPassword())).hint
