@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -18,22 +18,26 @@
 
 import logging
 
+import telectron
 from telectron import raw
-from telectron.scaffold import Scaffold
 
 log = logging.getLogger(__name__)
 
 
-class SendRecoveryCode(Scaffold):
-    async def send_recovery_code(self) -> str:
+class SendRecoveryCode:
+    async def send_recovery_code(
+        self: "telectron.Client",
+    ) -> str:
         """Send a code to your email to recover your password.
 
+        .. include:: /_includes/usable-by/users.rst
+
         Returns:
             ``str``: On success, the hidden email pattern is returned and a recovery code is sent to that email.
 
         Raises:
             BadRequest: In case no recovery email was set up.
         """
-        return (await self.send(
+        return (await self.invoke(
             raw.functions.auth.RequestPasswordRecovery()
         )).email_pattern
