@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -18,13 +18,20 @@
 
 from .answer_callback_query import AnswerCallbackQuery
 from .answer_inline_query import AnswerInlineQuery
+from .answer_web_app_query import AnswerWebAppQuery
+from .delete_bot_commands import DeleteBotCommands
+from .get_bot_commands import GetBotCommands
+from .get_bot_default_privileges import GetBotDefaultPrivileges
+from .get_chat_menu_button import GetChatMenuButton
 from .get_game_high_scores import GetGameHighScores
 from .get_inline_bot_results import GetInlineBotResults
 from .request_callback_answer import RequestCallbackAnswer
 from .send_game import SendGame
 from .send_inline_bot_result import SendInlineBotResult
-from .set_game_score import SetGameScore
 from .set_bot_commands import SetBotCommands
+from .set_bot_default_privileges import SetBotDefaultPrivileges
+from .set_chat_menu_button import SetChatMenuButton
+from .set_game_score import SetGameScore
 
 
 class Bots(
@@ -36,6 +43,13 @@ class Bots(
     SendGame,
     SetGameScore,
     GetGameHighScores,
-    SetBotCommands
+    SetBotCommands,
+    GetBotCommands,
+    DeleteBotCommands,
+    SetBotDefaultPrivileges,
+    GetBotDefaultPrivileges,
+    SetChatMenuButton,
+    GetChatMenuButton,
+    AnswerWebAppQuery
 ):
     pass
