@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -18,17 +18,19 @@
 
 from typing import Union
 
-from telectron.raw import functions
-from telectron.scaffold import Scaffold
+import telectron
+from telectron import raw
 
 
-class MarkChatUnread(Scaffold):
+class MarkChatUnread:
     async def mark_chat_unread(
-        self,
+        self: "telectron.Client",
         chat_id: Union[int, str],
     ) -> bool:
         """Mark a chat as unread.
 
+        .. include:: /_includes/usable-by/users.rst
+
         Parameters:
             chat_id (``int`` | ``str``):
                 Unique identifier (int) or username (str) of the target chat.
@@ -37,8 +39,8 @@ class MarkChatUnread(Scaffold):
             ``bool``: On success, True is returned.
         """
 
-        return await self.send(
-            functions.messages.MarkDialogUnread(
+        return await self.invoke(
+            raw.functions.messages.MarkDialogUnread(
                 peer=await self.resolve_peer(chat_id),
                 unread=True
             )
