@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -16,22 +16,26 @@
 #  You should have received a copy of the GNU Lesser General Public License
 #  along with telectron.  If not, see <http://www.gnu.org/licenses/>.
 
+import telectron
 from telectron import raw
-from telectron.scaffold import Scaffold
 
 
-class GetContactsCount(Scaffold):
-    async def get_contacts_count(self) -> int:
+class GetContactsCount:
+    async def get_contacts_count(
+        self: "telectron.Client"
+    ) -> int:
         """Get the total count of contacts from your Telegram address book.
 
+        .. include:: /_includes/usable-by/users.rst
+
         Returns:
             ``int``: On success, the contacts count is returned.
 
         Example:
             .. code-block:: python
 
-                count = app.get_contacts_count()
+                count = await app.get_contacts_count()
                 print(count)
         """
 
-        return len((await self.send(raw.functions.contacts.GetContacts(hash=0))).contacts)
+        return len((await self.invoke(raw.functions.contacts.GetContacts(hash=0))).contacts)
