@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -19,11 +19,10 @@
 from typing import Callable
 
 import telectron
-from telectron.scaffold import Scaffold
 
 
-class OnDisconnect(Scaffold):
-    def on_disconnect(self=None) -> callable:
+class OnDisconnect:
+    def on_disconnect(self=None) -> Callable:
         """Decorator for handling disconnections.
 
         This does the same thing as :meth:`~telectron.Client.add_handler` using the
@@ -33,6 +32,11 @@ class OnDisconnect(Scaffold):
         def decorator(func: Callable) -> Callable:
             if isinstance(self, telectron.Client):
                 self.add_handler(telectron.handlers.DisconnectHandler(func))
+            else:
+                if not hasattr(func, "handlers"):
+                    func.handlers = []
+
+                func.handlers.append((telectron.handlers.DisconnectHandler(func), 0))
 
             return func
 
