@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -19,21 +19,22 @@
 import logging
 import os
 import re
+from datetime import datetime
 from typing import Union, List
 
+import telectron
 from telectron import raw
 from telectron import types
 from telectron import utils
 from telectron.file_id import FileType
-from telectron.scaffold import Scaffold
 
 log = logging.getLogger(__name__)
 
 
-class SendMediaGroup(Scaffold):
+class SendMediaGroup:
     # TODO: Add progress parameter
     async def send_media_group(
-        self,
+        self: "telectron.Client",
         chat_id: Union[int, str],
         media: List[Union[
             "types.InputMediaPhoto",
@@ -41,12 +42,17 @@ class SendMediaGroup(Scaffold):
             "types.InputMediaAudio",
             "types.InputMediaDocument"
         ]],
+        file_names: List[str] = None,
         disable_notification: bool = None,
         reply_to_message_id: int = None,
-        schedule_date: int = None,
+        schedule_date: datetime = None,
+        protect_content: bool = None,
+        nosound_video: bool = None,
     ) -> List["types.Message"]:
         """Send a group of photos or videos as an album.
 
+        .. include:: /_includes/usable-by/users-bots.rst
+
         Parameters:
             chat_id (``int`` | ``str``):
                 Unique identifier (int) or username (str) of the target chat.
@@ -63,8 +69,11 @@ class SendMediaGroup(Scaffold):
             reply_to_message_id (``int``, *optional*):
                 If the message is a reply, ID of the original message.
 
-            schedule_date (``int``, *optional*):
-                Date when the message will be automatically sent. Unix time.
+            schedule_date (:py:obj:`~datetime.datetime`, *optional*):
+                Date when the message will be automatically sent.
+
+            protect_content (``bool``, *optional*):
+                Protects the contents of the sent message from forwarding and saving.
 
         Returns:
             List of :obj:`~telectron.types.Message`: On success, a list of the sent messages is returned.
@@ -74,26 +83,29 @@ class SendMediaGroup(Scaffold):
 
                 from telectron.types import InputMediaPhoto, InputMediaVideo
 
-                app.send_media_group(
+                await app.send_media_group(
                     "me",
                     [
                         InputMediaPhoto("photo1.jpg"),
                         InputMediaPhoto("photo2.jpg", caption="photo caption"),
-                        InputMediaVideo("video.mp4", caption="a video")
+                        InputMediaVideo("video.mp4", caption="video caption")
                     ]
                 )
         """
+        if file_names is None:
+            file_names = [object] * 10
         multi_media = []
 
-        for i in media:
+        for e, i in enumerate(media):
             if isinstance(i, types.InputMediaPhoto):
                 if isinstance(i.media, str):
                     if os.path.isfile(i.media):
-                        media = await self.send(
+                        media = await self.invoke(
                             raw.functions.messages.UploadMedia(
                                 peer=await self.resolve_peer(chat_id),
                                 media=raw.types.InputMediaUploadedPhoto(
-                                    file=await self.save_file(i.media)
+                                    file=await self.save_file(i.media),
+                                    spoiler=i.has_spoiler
                                 )
                             )
                         )
@@ -103,14 +115,16 @@ class SendMediaGroup(Scaffold):
                                 id=media.photo.id,
                                 access_hash=media.photo.access_hash,
                                 file_reference=media.photo.file_reference
-                            )
+                            ),
+                            spoiler=i.has_spoiler
                         )
                     elif re.match("^https?://", i.media):
-                        media = await self.send(
+                        media = await self.invoke(
                             raw.functions.messages.UploadMedia(
                                 peer=await self.resolve_peer(chat_id),
                                 media=raw.types.InputMediaPhotoExternal(
-                                    url=i.media
+                                    url=i.media,
+                                    spoiler=i.has_spoiler
                                 )
                             )
                         )
@@ -120,16 +134,18 @@ class SendMediaGroup(Scaffold):
                                 id=media.photo.id,
                                 access_hash=media.photo.access_hash,
                                 file_reference=media.photo.file_reference
-                            )
+                            ),
+                            spoiler=i.has_spoiler
                         )
                     else:
-                        media = utils.get_input_media_from_file_id(i.media, FileType.PHOTO)
+                        media = utils.get_input_media_from_file_id(i.media, FileType.PHOTO, spoiler=i.has_spoiler)
                 else:
-                    media = await self.send(
+                    media = await self.invoke(
                         raw.functions.messages.UploadMedia(
                             peer=await self.resolve_peer(chat_id),
                             media=raw.types.InputMediaUploadedPhoto(
-                                file=await self.save_file(i.media)
+                                file=await self.save_file(i.media),
+                                spoiler=i.has_spoiler
                             )
                         )
                     )
@@ -139,17 +155,19 @@ class SendMediaGroup(Scaffold):
                             id=media.photo.id,
                             access_hash=media.photo.access_hash,
                             file_reference=media.photo.file_reference
-                        )
+                        ),
+                        spoiler=i.has_spoiler
                     )
             elif isinstance(i, types.InputMediaVideo):
                 if isinstance(i.media, str):
                     if os.path.isfile(i.media):
-                        media = await self.send(
+                        media = await self.invoke(
                             raw.functions.messages.UploadMedia(
                                 peer=await self.resolve_peer(chat_id),
                                 media=raw.types.InputMediaUploadedDocument(
                                     file=await self.save_file(i.media),
                                     thumb=await self.save_file(i.thumb),
+                                    spoiler=i.has_spoiler,
                                     mime_type=self.guess_mime_type(i.media) or "video/mp4",
                                     attributes=[
                                         raw.types.DocumentAttributeVideo(
@@ -158,8 +176,12 @@ class SendMediaGroup(Scaffold):
                                             w=i.width,
                                             h=i.height
                                         ),
-                                        raw.types.DocumentAttributeFilename(file_name=os.path.basename(i.media))
-                                    ]
+                                        raw.types.DocumentAttributeFilename(
+                                            file_name=file_names[e] if file_names[e] is not object
+                                            else os.path.basename(i.media)
+                                        )
+                                    ],
+                                    nosound_video=nosound_video,
                                 )
                             )
                         )
@@ -169,14 +191,16 @@ class SendMediaGroup(Scaffold):
                                 id=media.document.id,
                                 access_hash=media.document.access_hash,
                                 file_reference=media.document.file_reference
-                            )
+                            ),
+                            spoiler=i.has_spoiler
                         )
                     elif re.match("^https?://", i.media):
-                        media = await self.send(
+                        media = await self.invoke(
                             raw.functions.messages.UploadMedia(
                                 peer=await self.resolve_peer(chat_id),
                                 media=raw.types.InputMediaDocumentExternal(
-                                    url=i.media
+                                    url=i.media,
+                                    spoiler=i.has_spoiler
                                 )
                             )
                         )
@@ -186,27 +210,33 @@ class SendMediaGroup(Scaffold):
                                 id=media.document.id,
                                 access_hash=media.document.access_hash,
                                 file_reference=media.document.file_reference
-                            )
+                            ),
+                            spoiler=i.has_spoiler
                         )
                     else:
-                        media = utils.get_input_media_from_file_id(i.media, FileType.VIDEO)
+                        media = utils.get_input_media_from_file_id(i.media, FileType.VIDEO, spoiler=i.has_spoiler)
                 else:
-                    media = await self.send(
+                    attributes = [raw.types.DocumentAttributeVideo(
+                        supports_streaming=i.supports_streaming or None,
+                        duration=i.duration,
+                        w=i.width,
+                        h=i.height
+                    )]
+                    if file_names[e] is not None:
+                        attributes.append(raw.types.DocumentAttributeFilename(
+                            file_name=file_names[e] if file_names[e] is not object
+                            else getattr(i.media, "name", "video.mp4")
+                        ))
+                    media = await self.invoke(
                         raw.functions.messages.UploadMedia(
                             peer=await self.resolve_peer(chat_id),
                             media=raw.types.InputMediaUploadedDocument(
                                 file=await self.save_file(i.media),
                                 thumb=await self.save_file(i.thumb),
+                                spoiler=i.has_spoiler,
                                 mime_type=self.guess_mime_type(getattr(i.media, "name", "video.mp4")) or "video/mp4",
-                                attributes=[
-                                    raw.types.DocumentAttributeVideo(
-                                        supports_streaming=i.supports_streaming or None,
-                                        duration=i.duration,
-                                        w=i.width,
-                                        h=i.height
-                                    ),
-                                    raw.types.DocumentAttributeFilename(file_name=getattr(i.media, "name", "video.mp4"))
-                                ]
+                                attributes=attributes,
+                                nosound_video=nosound_video,
                             )
                         )
                     )
@@ -216,12 +246,13 @@ class SendMediaGroup(Scaffold):
                             id=media.document.id,
                             access_hash=media.document.access_hash,
                             file_reference=media.document.file_reference
-                        )
+                        ),
+                        spoiler=i.has_spoiler
                     )
             elif isinstance(i, types.InputMediaAudio):
                 if isinstance(i.media, str):
                     if os.path.isfile(i.media):
-                        media = await self.send(
+                        media = await self.invoke(
                             raw.functions.messages.UploadMedia(
                                 peer=await self.resolve_peer(chat_id),
                                 media=raw.types.InputMediaUploadedDocument(
@@ -234,7 +265,10 @@ class SendMediaGroup(Scaffold):
                                             performer=i.performer,
                                             title=i.title
                                         ),
-                                        raw.types.DocumentAttributeFilename(file_name=os.path.basename(i.media))
+                                        raw.types.DocumentAttributeFilename(
+                                            file_name=file_names[e] if file_names[e] is not object
+                                            else os.path.basename(i.media)
+                                        )
                                     ]
                                 )
                             )
@@ -248,7 +282,7 @@ class SendMediaGroup(Scaffold):
                             )
                         )
                     elif re.match("^https?://", i.media):
-                        media = await self.send(
+                        media = await self.invoke(
                             raw.functions.messages.UploadMedia(
                                 peer=await self.resolve_peer(chat_id),
                                 media=raw.types.InputMediaDocumentExternal(
@@ -267,7 +301,7 @@ class SendMediaGroup(Scaffold):
                     else:
                         media = utils.get_input_media_from_file_id(i.media, FileType.AUDIO)
                 else:
-                    media = await self.send(
+                    media = await self.invoke(
                         raw.functions.messages.UploadMedia(
                             peer=await self.resolve_peer(chat_id),
                             media=raw.types.InputMediaUploadedDocument(
@@ -280,7 +314,10 @@ class SendMediaGroup(Scaffold):
                                         performer=i.performer,
                                         title=i.title
                                     ),
-                                    raw.types.DocumentAttributeFilename(file_name=getattr(i.media, "name", "audio.mp3"))
+                                    raw.types.DocumentAttributeFilename(
+                                        file_name=(file_names[e] if file_names[e] is not object else None)
+                                                  or getattr(i.media, "name", "audio.mp3")
+                                    )
                                 ]
                             )
                         )
@@ -296,7 +333,7 @@ class SendMediaGroup(Scaffold):
             elif isinstance(i, types.InputMediaDocument):
                 if isinstance(i.media, str):
                     if os.path.isfile(i.media):
-                        media = await self.send(
+                        media = await self.invoke(
                             raw.functions.messages.UploadMedia(
                                 peer=await self.resolve_peer(chat_id),
                                 media=raw.types.InputMediaUploadedDocument(
@@ -304,7 +341,10 @@ class SendMediaGroup(Scaffold):
                                     file=await self.save_file(i.media),
                                     thumb=await self.save_file(i.thumb),
                                     attributes=[
-                                        raw.types.DocumentAttributeFilename(file_name=os.path.basename(i.media))
+                                        raw.types.DocumentAttributeFilename(
+                                            file_name=file_names[e] if file_names[e] is not object
+                                            else os.path.basename(i.media)
+                                        )
                                     ]
                                 )
                             )
@@ -318,7 +358,7 @@ class SendMediaGroup(Scaffold):
                             )
                         )
                     elif re.match("^https?://", i.media):
-                        media = await self.send(
+                        media = await self.invoke(
                             raw.functions.messages.UploadMedia(
                                 peer=await self.resolve_peer(chat_id),
                                 media=raw.types.InputMediaDocumentExternal(
@@ -337,7 +377,7 @@ class SendMediaGroup(Scaffold):
                     else:
                         media = utils.get_input_media_from_file_id(i.media, FileType.DOCUMENT)
                 else:
-                    media = await self.send(
+                    media = await self.invoke(
                         raw.functions.messages.UploadMedia(
                             peer=await self.resolve_peer(chat_id),
                             media=raw.types.InputMediaUploadedDocument(
@@ -347,7 +387,10 @@ class SendMediaGroup(Scaffold):
                                 file=await self.save_file(i.media),
                                 thumb=await self.save_file(i.thumb),
                                 attributes=[
-                                    raw.types.DocumentAttributeFilename(file_name=getattr(i.media, "name", "file.zip"))
+                                    raw.types.DocumentAttributeFilename(
+                                        file_name=(file_names[e] if file_names[e] is not object else None)
+                                                  or getattr(i.media, "name", "file.zip")
+                                    )
                                 ]
                             )
                         )
@@ -371,13 +414,15 @@ class SendMediaGroup(Scaffold):
                 )
             )
 
-        r = await self.send(
+        r = await self.invoke(
             raw.functions.messages.SendMultiMedia(
                 peer=await self.resolve_peer(chat_id),
                 multi_media=multi_media,
                 silent=disable_notification or None,
-                reply_to_msg_id=reply_to_message_id,
-                schedule_date=schedule_date
+                reply_to=(None if reply_to_message_id is None else
+                          raw.types.InputReplyToMessage(reply_to_msg_id=reply_to_message_id)),
+                schedule_date=utils.datetime_to_timestamp(schedule_date),
+                noforwards=protect_content
             ),
             sleep_threshold=60
         )
