@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -16,29 +16,34 @@
 #  You should have received a copy of the GNU Lesser General Public License
 #  along with telectron.  If not, see <http://www.gnu.org/licenses/>.
 
+import telectron
 from telectron import raw
 from telectron import types
-from telectron.scaffold import Scaffold
 
 
-class GetMe(Scaffold):
-    async def get_me(self) -> "types.User":
+class GetMe:
+    async def get_me(
+        self: "telectron.Client"
+    ) -> "types.User":
         """Get your own user identity.
 
+        .. include:: /_includes/usable-by/users-bots.rst
+
         Returns:
             :obj:`~telectron.types.User`: Information about the own logged in user/bot.
 
         Example:
             .. code-block:: python
 
-                me = app.get_me()
+                me = await app.get_me()
                 print(me)
         """
-        return types.User._parse(
-            self,
-            (await self.send(
-                raw.functions.users.GetFullUser(
-                    id=raw.types.InputUserSelf()
-                )
-            )).user
+        r = await self.invoke(
+            raw.functions.users.GetFullUser(
+                id=raw.types.InputUserSelf()
+            )
         )
+
+        users = {u.id: u for u in r.users}
+
+        return types.User._parse(self, users[r.full_user.id])
