@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -21,6 +21,7 @@ import re
 from typing import Optional
 
 import telectron
+from telectron.enums import MessageEntityType
 from . import utils
 from .html import HTML
 
@@ -28,6 +29,7 @@ BOLD_DELIM = "**"
 ITALIC_DELIM = "__"
 UNDERLINE_DELIM = "--"
 STRIKE_DELIM = "~~"
+SPOILER_DELIM = "||"
 CODE_DELIM = "`"
 PRE_DELIM = "```"
 
@@ -41,7 +43,8 @@ MARKDOWN_RE = re.compile(r"({d})|\[(.+?)\]\((.+?)\)".format(
                 STRIKE_DELIM,
                 UNDERLINE_DELIM,
                 ITALIC_DELIM,
-                BOLD_DELIM
+                BOLD_DELIM,
+                SPOILER_DELIM
             ]
         ]]
     )))
@@ -90,6 +93,8 @@ class Markdown:
                 tag = "code"
             elif delim == PRE_DELIM:
                 tag = "pre"
+            elif delim == SPOILER_DELIM:
+                tag = "spoiler"
             else:
                 continue
 
@@ -100,6 +105,12 @@ class Markdown:
                 delims.remove(delim)
                 tag = CLOSING_TAG.format(tag)
 
+            if delim == PRE_DELIM and delim in delims:
+                delim_and_language = text[text.find(PRE_DELIM):].split("\n")[0]
+                language = delim_and_language[len(PRE_DELIM):]
+                text = utils.replace_once(text, delim_and_language, f'<pre language="{language}">', start)
+                continue
+
             text = utils.replace_once(text, delim, tag, start)
 
         return await self.html.parse(text)
@@ -111,29 +122,33 @@ class Markdown:
         entities_offsets = []
 
         for entity in entities:
-            if entity is None:
-                continue
             entity_type = entity.type
             start = entity.offset
             end = start + entity.length
 
-            if entity_type == "bold":
+            if entity_type == MessageEntityType.BOLD:
                 start_tag = end_tag = BOLD_DELIM
-            elif entity_type == "italic":
+            elif entity_type == MessageEntityType.ITALIC:
                 start_tag = end_tag = ITALIC_DELIM
-            elif entity_type == "underline":
+            elif entity_type == MessageEntityType.UNDERLINE:
                 start_tag = end_tag = UNDERLINE_DELIM
-            elif entity_type == "strikethrough":
+            elif entity_type == MessageEntityType.STRIKETHROUGH:
                 start_tag = end_tag = STRIKE_DELIM
-            elif entity_type == "code":
+            elif entity_type == MessageEntityType.CODE:
                 start_tag = end_tag = CODE_DELIM
-            elif entity_type in ("pre", "blockquote"):
+            elif entity_type == MessageEntityType.PRE:
+                language = getattr(entity, "language", "") or ""
+                start_tag = f"{PRE_DELIM}{language}\n"
+                end_tag = f"\n{PRE_DELIM}"
+            elif entity_type == MessageEntityType.BLOCKQUOTE:
                 start_tag = end_tag = PRE_DELIM
-            elif entity_type == "text_link":
+            elif entity_type == MessageEntityType.SPOILER:
+                start_tag = end_tag = SPOILER_DELIM
+            elif entity_type == MessageEntityType.TEXT_LINK:
                 url = entity.url
                 start_tag = "["
                 end_tag = f"]({url})"
-            elif entity_type == "text_mention":
+            elif entity_type == MessageEntityType.TEXT_MENTION:
                 user = entity.user
                 start_tag = "["
                 end_tag = f"](tg://user?id={user.id})"
@@ -143,8 +158,14 @@ class Markdown:
             entities_offsets.append((start_tag, start,))
             entities_offsets.append((end_tag, end,))
 
-        # sorting by offset (desc)
-        entities_offsets.sort(key=lambda x: -x[1])
+        entities_offsets = map(
+            lambda x: x[1],
+            sorted(
+                enumerate(entities_offsets),
+                key=lambda x: (x[1][1], x[0]),
+                reverse=True
+            )
+        )
 
         for entity, offset in entities_offsets:
             text = text[:offset] + entity + text[offset:]
