@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -23,4 +23,4 @@ from .tl_object import TLObject
 
 class List(TList[Any], TLObject):
     def __repr__(self) -> str:
-        return f"telectron.api.core.List([{','.join(TLObject.__repr__(i) for i in self)}])"
+        return f"telectron.raw.core.List([{','.join(TLObject.__repr__(i) for i in self)}])"
