@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -17,29 +17,19 @@
 #  along with telectron.  If not, see <http://www.gnu.org/licenses/>.
 
 import logging
-from datetime import datetime
-from time import perf_counter
+import time
 
 log = logging.getLogger(__name__)
 
 
 class MsgId:
-    reference_clock = 0
     last_time = 0
-    msg_id_offset = 0
-    server_time = 0
+    offset = 0
 
     def __new__(cls) -> int:
-        now = perf_counter() - cls.reference_clock + cls.server_time
-        cls.msg_id_offset = cls.msg_id_offset + 4 if now == cls.last_time else 0
-        msg_id = int(now * 2 ** 32) + cls.msg_id_offset
+        now = int(time.time())
+        cls.offset = (cls.offset + 4) if now == cls.last_time else 0
+        msg_id = (now * 2 ** 32) + cls.offset
         cls.last_time = now
 
         return msg_id
-
-    @classmethod
-    def set_server_time(cls, server_time: int):
-        if not cls.server_time:
-            cls.reference_clock = perf_counter()
-            cls.server_time = server_time
-            log.info(f"Time synced: {datetime.utcfromtimestamp(server_time)} UTC")
