@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -16,19 +16,15 @@
 #  You should have received a copy of the GNU Lesser General Public License
 #  along with telectron.  If not, see <http://www.gnu.org/licenses/>.
 
-from threading import Lock
-
 
 class SeqNo:
     def __init__(self):
         self.content_related_messages_sent = 0
-        self.lock = Lock()
 
     def __call__(self, is_content_related: bool) -> int:
-        with self.lock:
-            seq_no = (self.content_related_messages_sent * 2) + (1 if is_content_related else 0)
+        seq_no = (self.content_related_messages_sent * 2) + (1 if is_content_related else 0)
 
-            if is_content_related:
-                self.content_related_messages_sent += 1
+        if is_content_related:
+            self.content_related_messages_sent += 1
 
-            return seq_no
+        return seq_no
