@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -18,7 +18,6 @@
 
 import sqlite3
 import time
-from threading import Lock
 from typing import List, Tuple, Any
 
 from telectron import raw
@@ -30,6 +29,7 @@ SCHEMA = """
 CREATE TABLE sessions
 (
     dc_id     INTEGER PRIMARY KEY,
+    api_id    INTEGER,
     test_mode INTEGER,
     auth_key  BLOB,
     date      INTEGER NOT NULL,
@@ -89,17 +89,16 @@ def get_input_peer(peer_id: int, access_hash: int, peer_type: str):
 
 
 class SQLiteStorage(Storage):
-    VERSION = 2
+    VERSION = 3
     USERNAME_TTL = 8 * 60 * 60
 
     def __init__(self, name: str):
         super().__init__(name)
 
         self.conn = None  # type: sqlite3.Connection
-        self.lock = Lock()
 
     def create(self):
-        with self.lock, self.conn:
+        with self.conn:
             self.conn.executescript(SCHEMA)
 
             self.conn.execute(
@@ -108,8 +107,8 @@ class SQLiteStorage(Storage):
             )
 
             self.conn.execute(
-                "INSERT INTO sessions VALUES (?, ?, ?, ?, ?, ?)",
-                (2, None, None, 0, None, None)
+                "INSERT INTO sessions VALUES (?, ?, ?, ?, ?, ?, ?)",
+                (2, None, None, None, 0, None, None)
             )
 
     async def open(self):
@@ -117,24 +116,20 @@ class SQLiteStorage(Storage):
 
     async def save(self):
         await self.date(int(time.time()))
-
-        with self.lock:
-            self.conn.commit()
+        self.conn.commit()
 
     async def close(self):
-        with self.lock:
-            self.conn.close()
+        self.conn.close()
 
     async def delete(self):
         raise NotImplementedError
 
     async def update_peers(self, peers: List[Tuple[int, int, str, str, str]]):
-        with self.lock:
-            self.conn.executemany(
-                "REPLACE INTO peers (id, access_hash, type, username, phone_number)"
-                "VALUES (?, ?, ?, ?, ?)",
-                peers
-            )
+        self.conn.executemany(
+            "REPLACE INTO peers (id, access_hash, type, username, phone_number)"
+            "VALUES (?, ?, ?, ?, ?)",
+            peers
+        )
 
     async def get_peer_by_id(self, peer_id: int):
         r = self.conn.execute(
@@ -149,7 +144,8 @@ class SQLiteStorage(Storage):
 
     async def get_peer_by_username(self, username: str):
         r = self.conn.execute(
-            "SELECT id, access_hash, type, last_update_on FROM peers WHERE username = ?",
+            "SELECT id, access_hash, type, last_update_on FROM peers WHERE username = ?"
+            "ORDER BY last_update_on DESC",
             (username,)
         ).fetchone()
 
@@ -172,38 +168,41 @@ class SQLiteStorage(Storage):
 
         return get_input_peer(*r)
 
-    def _get(self, attr):
+    def _get(self, attr: str):
         return self.conn.execute(
             f"SELECT {attr} FROM sessions"
         ).fetchone()[0]
 
-    def _set(self, attr, value: Any):
-        with self.lock, self.conn:
+    def _set(self, attr: str, value: Any):
+        with self.conn:
             self.conn.execute(
                 f"UPDATE sessions SET {attr} = ?",
                 (value,)
             )
 
-    def _accessor(self, attr, value: Any = object):
+    def _accessor(self, attr: str, value: Any = object):
         return self._get(attr) if value == object else self._set(attr, value)
 
     async def dc_id(self, value: int = object):
-        return self._accessor('dc_id', value)
+        return self._accessor("dc_id", value)
+
+    async def api_id(self, value: int = object):
+        return self._accessor("api_id", value)
 
     async def test_mode(self, value: bool = object):
-        return self._accessor('test_mode', value)
+        return self._accessor("test_mode", value)
 
     async def auth_key(self, value: bytes = object):
-        return self._accessor('auth_key', value)
+        return self._accessor("auth_key", value)
 
     async def date(self, value: int = object):
-        return self._accessor('date', value)
+        return self._accessor("date", value)
 
     async def user_id(self, value: int = object):
-        return self._accessor('user_id', value)
+        return self._accessor("user_id", value)
 
     async def is_bot(self, value: bool = object):
-        return self._accessor('is_bot', value)
+        return self._accessor("is_bot", value)
 
     def version(self, value: int = object):
         if value == object:
@@ -211,7 +210,7 @@ class SQLiteStorage(Storage):
                 "SELECT number FROM version"
             ).fetchone()[0]
         else:
-            with self.lock, self.conn:
+            with self.conn:
                 self.conn.execute(
                     "UPDATE version SET number = ?",
                     (value,)
