@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -17,6 +17,7 @@
 #  along with telectron.  If not, see <http://www.gnu.org/licenses/>.
 
 from telectron import raw
+
 from ..object import Object
 
 
@@ -25,10 +26,11 @@ class BotCommand(Object):
 
     Parameters:
         command (``str``):
-            The bot command, for example: "/start".
-            
+            Text of the command; 1-32 characters.
+            Can contain only lowercase English letters, digits and underscores.
+
         description (``str``):
-            Description of the bot command.
+            Description of the command; 1-256 characters.
     """
 
     def __init__(self, command: str, description: str):
@@ -37,8 +39,15 @@ class BotCommand(Object):
         self.command = command
         self.description = description
 
-    def write(self):
+    def write(self) -> "raw.types.BotCommand":
         return raw.types.BotCommand(
             command=self.command,
-            description=self.description
+            description=self.description,
+        )
+
+    @staticmethod
+    def read(c: "raw.types.BotCommand") -> "BotCommand":
+        return BotCommand(
+            command=c.command,
+            description=c.description
         )
