@@ -1,5 +1,5 @@
 #  telectron - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of telectron.
 #
@@ -16,238 +16,126 @@
 #  You should have received a copy of the GNU Lesser General Public License
 #  along with telectron.  If not, see <http://www.gnu.org/licenses/>.
 
-from enum import Enum, auto
+from datetime import datetime
 from typing import List, Optional
 
 import telectron
 from telectron import raw
-from telectron import types
+from telectron import types, utils, enums
 from ..object import Object
 
 
-class AutoName(Enum):
-    def _generate_next_value_(self, *args):
-        return self.lower()
-
-
-class ChatEventAction(AutoName):
-    DESCRIPTION_CHANGED = auto()
-    HISTORY_TTL_CHANGED = auto()
-    LINKED_CHAT_CHANGED = auto()
-    # LOCATION_CHANGED = auto()
-    PHOTO_CHANGED = auto()
-    # STICKER_SET_CHANGED = auto()
-    TITLE_CHANGED = auto()
-    USERNAME_CHANGED = auto()
-    CHAT_PERMISSIONS_CHANGED = auto()
-    MESSAGE_DELETED = auto()
-    # VOICE_CHAT_DISCARDED = auto()
-    MESSAGE_EDITED = auto()
-    INVITE_LINK_EDITED = auto()
-    INVITE_LINK_REVOKED = auto()
-    INVITE_LINK_DELETED = auto()
-    MEMBER_INVITED = auto()
-    MEMBER_JOINED = auto()
-    # MEMBER_JOINED_BY_LINK = auto()
-    MEMBER_LEFT = auto()
-    # MEMBER_MUTED = auto()
-    ADMIN_RIGHTS_CHANGED = auto()
-    MEMBER_PERMISSIONS_CHANGED = auto()
-    # MEMBER_UNMUTED = auto()
-    # MEMBER_VOLUME_CHANGED = auto()
-    # VOICE_CHAT_STARTED = auto()
-    POLL_STOPPED = auto()
-    # VOICE_CHAT_SETTINGS_CHANGED = auto()
-    INVITES_ENABLED = auto()
-    HISTORY_HIDDEN = auto()
-    SIGNATURES_ENABLED = auto()
-    SLOW_MODE_CHANGED = auto()
-    MESSAGE_PINNED = auto()
-    MESSAGE_UNPINNED = auto()
-    UNKNOWN = auto()
-
-
 class ChatEvent(Object):
     """A chat event from the recent actions log (also known as admin log).
 
+    See ``action`` to know which kind of event this is and the relative attributes to get the event content.
+
     Parameters:
         id (``int``):
             Chat event identifier.
 
-        date (``int``):
-            Date of the event. Unix time.
+        date (:py:obj:`~datetime.datetime`):
+            Date of the event.
 
-        action (``str``):
-            Event action. Can be:
-
-            - "description_changed": the chat description has been changed
-              (see *old_description* and *new_description* below).
-
-            - "history_ttl_changed": the history time-to-live has been changed
-              (see *old_history_ttl* and *new_history_ttl* below).
-
-            - "linked_chat_changed": the linked chat has been changed
-              (see *old_linked_chat* and *new_linked_chat* below).
-
-            - "photo_changed": the chat photo has been changed
-              (see *old_photo* and *new_photo* below).
-
-            - "title_changed": the chat title has been changed
-              (see *old_title* and *new_title* below).
-
-            - "username_changed": the chat username has been changed
-              (see *old_username* and *new_username* below).
-
-            - "chat_permissions_changed": the default chat permissions has been changed
-              (see *old_chat_permissions* and *new_chat_permissions* below).
-
-            - "message_deleted": a message has been deleted
-              (see *deleted_message* below).
-
-            - "message_edited": a message has been edited
-              (see *old_message* and *new_message* below).
-
-            - "member_invited": a member has been invited by someone
-              (see *invited_member* below).
-
-            - "member_joined": a member joined by themselves.
-              (see *user* below)
-
-            - "member_left": a member left by themselves.
-              (see *user* below).
-
-            - "admin_rights_changed": a chat member has been promoted/demoted or their administrator rights has changed
-              (see *old_admin_rights* and *new_admin_rights* below).
-
-            - "member_permissions_changed": a chat member has been restricted/unrestricted or banned/unbanned, or their
-              permissions has changed (see *old_member_permissions* and *new_member_permissions* below).
-
-            - "poll_stopped": a poll has been stopped
-              (see *stopped_poll* below).
-
-            - "invites_enabled": the chat invitation has been enabled or disabled
-              (see *invites_enabled* below).
-
-            - "history_hidden": the chat history has been hidden or unhidden
-              (see *history_hidden* below).
-
-            - "signatures_enabled": the message signatures have been enabled or disabled
-              (see *signatures_enabled* below).
-
-            - "slow_mode_changed": the slow mode has been changes
-              (see *old_slow_mode* and *new_slow_mode* below).
-
-            - "message_pinned": a message has been pinned
-              (see *pinned_message* below).
-
-            - "message_unpinned": a message has been unpinned
-              (see *unpinned_message* below).
-
-            - "invite_link_edited": an invite link has been edited
-              (see *edited_invite_link* below).
-
-            - "invite_link_revoked": an invite link has been revoked
-              (see *revoked_invite_link* below).
-
-            - "invite_link_deleted": an invite link has been deleted
-              (see *deleted_invite_link* below).
+        action (:obj:`~telectron.enums.ChatEventAction`):
+            Event action.
 
         user (:obj:`~telectron.types.User`):
             User that triggered the event.
 
         old_description, new_description (``str``, *optional*):
             Previous and new chat description.
-            For "description_changed" only.
+            For :obj:`~telectron.enums.ChatEventAction.DESCRIPTION_CHANGED` action only.
 
         old_history_ttl, new_history_ttl (``int``, *optional*):
             Previous and new chat history TTL.
-            For "history_ttl_changed" only.
+            For :obj:`~telectron.enums.ChatEventAction.HISTORY_TTL_CHANGED` action only.
 
         old_linked_chat, new_linked_chat (:obj:`~telectron.types.Chat`, *optional*):
             Previous and new linked chat.
-            For "linked_chat_changed" only.
+            For :obj:`~telectron.enums.ChatEventAction.LINKED_CHAT_CHANGED` action only.
 
         old_photo, new_photo (:obj:`~telectron.types.Photo`, *optional*):
             Previous and new chat photo.
-            For "photo_changed" only.
+            For :obj:`~telectron.enums.ChatEventAction.PHOTO_CHANGED` action only.
 
         old_title, new_title (``str``, *optional*):
             Previous and new chat title.
-            For "title_changed" only.
+            For :obj:`~telectron.enums.ChatEventAction.TITLE_CHANGED` action only.
 
         old_username, new_username (``str``, *optional*):
             Previous and new chat username.
-            For "username_changed" only.
+            For :obj:`~telectron.enums.ChatEventAction.USERNAME_CHANGED` action only.
 
         old_chat_permissions, new_chat_permissions (:obj:`~telectron.types.ChatPermissions`, *optional*):
             Previous and new default chat permissions.
-            For "chat_permissions_changed" only.
+            For :obj:`~telectron.enums.ChatEventAction.CHAT_PERMISSIONS_CHANGED` action only.
 
         deleted_message (:obj:`~telectron.types.Message`, *optional*):
             Deleted message.
-            For "deleted_message" only.
+            For :obj:`~telectron.enums.ChatEventAction.MESSAGE_DELETED` action only.
 
         old_message, new_message (:obj:`~telectron.types.Message`, *optional*):
             Previous and new message before it has been edited.
-            For "message_edited" only.
+            For :obj:`~telectron.enums.ChatEventAction.MESSAGE_EDITED` action only.
 
         invited_member (:obj:`~telectron.types.ChatMember`, *optional*):
             New invited chat member.
-            For "member_invited" only.
+            For :obj:`~telectron.enums.ChatEventAction.MEMBER_INVITED` action only.
 
-        old_admin_rights, new_admin_rights (:obj:`~telectron.types.ChatMember`, *optional*):
-            Previous and new administrator rights.
-            For "admin_rights_changed" only.
+        old_administrator_privileges, new_administrator_privileges (:obj:`~telectron.types.ChatMember`, *optional*):
+            Previous and new administrator privileges.
+            For :obj:`~telectron.enums.ChatEventAction.ADMINISTRATOR_PRIVILEGES_CHANGED` action only.
 
         old_member_permissions, new_member_permissions (:obj:`~telectron.types.ChatMember`, *optional*):
             Previous and new member permissions.
-            For "member_permissions_changed" only.
+            For :obj:`~telectron.enums.ChatEventAction.MEMBER_PERMISSIONS_CHANGED` action only.
 
         stopped_poll (:obj:`~telectron.types.Message`, *optional*):
             Message containing the stopped poll.
-            For "poll_stopped" only.
+            For :obj:`~telectron.enums.ChatEventAction.POLL_STOPPED` action only.
 
         invites_enabled (``bool``, *optional*):
             If chat invites were enabled (True) or disabled (False).
-            For "invites_enabled" only.
+            For :obj:`~telectron.enums.ChatEventAction.INVITES_ENABLED` action only.
 
         history_hidden (``bool``, *optional*):
             If chat history has been hidden (True) or unhidden (False).
-            For "history_hidden" only.
+            For :obj:`~telectron.enums.ChatEventAction.HISTORY_HIDDEN` action only.
 
         signatures_enabled (``bool``, *optional*):
             If message signatures were enabled (True) or disabled (False).
-            For "signatures_enabled" only.
+            For :obj:`~telectron.enums.ChatEventAction.SIGNATURES_ENABLED` action only.
 
         old_slow_mode, new_slow_mode (``int``, *optional*):
             Previous and new slow mode value in seconds.
-            For "slow_mode_changed" only.
+            For :obj:`~telectron.enums.ChatEventAction.SLOW_MODE_CHANGED` action only.
 
         pinned_message (:obj:`~telectron.types.Message`, *optional*):
             Pinned message.
-            For "message_pinned" only.
+            For :obj:`~telectron.enums.ChatEventAction.MESSAGE_PINNED` action only.
 
         unpinned_message (:obj:`~telectron.types.Message`, *optional*):
             Unpinned message.
-            For "unpinned_message" only.
+            For :obj:`~telectron.enums.ChatEventAction.MESSAGE_UNPINNED` action only.
 
         old_invite_link, new_invite_link (:obj:`~telectron.types.ChatInviteLink`, *optional*):
             Previous and new edited invite link.
-            For "invite_link_edited" only.
+            For :obj:`~telectron.enums.ChatEventAction.INVITE_LINK_EDITED` action only.
 
         revoked_invite_link (:obj:`~telectron.types.ChatInviteLink`, *optional*):
             Revoked invite link.
-            For "invite_link_revoked" only.
+            For :obj:`~telectron.enums.ChatEventAction.INVITE_LINK_REVOKED` action only.
 
         deleted_invite_link (:obj:`~telectron.types.ChatInviteLink`, *optional*):
             Deleted invite link.
-            For "invite_link_deleted" only.
+            For :obj:`~telectron.enums.ChatEventAction.INVITE_LINK_DELETED` action only.
     """
 
     def __init__(
         self, *,
         id: int,
-        date: int,
+        date: datetime,
         user: "types.User",
         action: str,
 
@@ -279,8 +167,8 @@ class ChatEvent(Object):
 
         invited_member: "types.ChatMember" = None,
 
-        old_admin_rights: "types.ChatMember" = None,
-        new_admin_rights: "types.ChatMember" = None,
+        old_administrator_privileges: "types.ChatMember" = None,
+        new_administrator_privileges: "types.ChatMember" = None,
 
         old_member_permissions: "types.ChatMember" = None,
         new_member_permissions: "types.ChatMember" = None,
@@ -302,7 +190,7 @@ class ChatEvent(Object):
         old_invite_link: "types.ChatInviteLink" = None,
         new_invite_link: "types.ChatInviteLink" = None,
         revoked_invite_link: "types.ChatInviteLink" = None,
-        deleted_invite_link: "types.ChatInviteLink" = None,
+        deleted_invite_link: "types.ChatInviteLink" = None
     ):
         super().__init__()
 
@@ -339,8 +227,8 @@ class ChatEvent(Object):
 
         self.invited_member = invited_member
 
-        self.old_admin_rights = old_admin_rights
-        self.new_admin_rights = new_admin_rights
+        self.old_administrator_privileges = old_administrator_privileges
+        self.new_administrator_privileges = new_administrator_privileges
 
         self.old_member_permissions = old_member_permissions
         self.new_member_permissions = new_member_permissions
@@ -405,8 +293,8 @@ class ChatEvent(Object):
 
         invited_member: Optional[types.ChatMember] = None
 
-        old_admin_rights: Optional[types.ChatMember] = None
-        new_admin_rights: Optional[types.ChatMember] = None
+        old_administrator_privileges: Optional[types.ChatMember] = None
+        new_administrator_privileges: Optional[types.ChatMember] = None
 
         old_member_permissions: Optional[types.ChatMember] = None
         new_member_permissions: Optional[types.ChatMember] = None
@@ -433,117 +321,117 @@ class ChatEvent(Object):
         if isinstance(action, raw.types.ChannelAdminLogEventActionChangeAbout):
             old_description = action.prev_value
             new_description = action.new_value
-            action = ChatEventAction.DESCRIPTION_CHANGED.value
+            action = enums.ChatEventAction.DESCRIPTION_CHANGED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionChangeHistoryTTL):
             old_history_ttl = action.prev_value
             new_history_ttl = action.new_value
-            action = ChatEventAction.HISTORY_TTL_CHANGED.value
+            action = enums.ChatEventAction.HISTORY_TTL_CHANGED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionChangeLinkedChat):
             old_linked_chat = types.Chat._parse_chat(client, chats[action.prev_value])
             new_linked_chat = types.Chat._parse_chat(client, chats[action.new_value])
-            action = ChatEventAction.LINKED_CHAT_CHANGED.value
+            action = enums.ChatEventAction.LINKED_CHAT_CHANGED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionChangePhoto):
             old_photo = types.Photo._parse(client, action.prev_photo)
             new_photo = types.Photo._parse(client, action.new_photo)
-            action = ChatEventAction.PHOTO_CHANGED.value
+            action = enums.ChatEventAction.PHOTO_CHANGED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionChangeTitle):
             old_title = action.prev_value
             new_title = action.new_value
-            action = ChatEventAction.TITLE_CHANGED.value
+            action = enums.ChatEventAction.TITLE_CHANGED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionChangeUsername):
             old_username = action.prev_value
             new_username = action.new_value
-            action = ChatEventAction.USERNAME_CHANGED.value
+            action = enums.ChatEventAction.USERNAME_CHANGED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionDefaultBannedRights):
             old_chat_permissions = types.ChatPermissions._parse(action.prev_banned_rights)
             new_chat_permissions = types.ChatPermissions._parse(action.new_banned_rights)
-            action = ChatEventAction.CHAT_PERMISSIONS_CHANGED.value
+            action = enums.ChatEventAction.CHAT_PERMISSIONS_CHANGED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionDeleteMessage):
             deleted_message = await types.Message._parse(client, action.message, users, chats)
-            action = ChatEventAction.MESSAGE_DELETED.value
+            action = enums.ChatEventAction.MESSAGE_DELETED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionEditMessage):
             old_message = await types.Message._parse(client, action.prev_message, users, chats)
             new_message = await types.Message._parse(client, action.new_message, users, chats)
-            action = ChatEventAction.MESSAGE_EDITED.value
+            action = enums.ChatEventAction.MESSAGE_EDITED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionParticipantInvite):
             invited_member = types.ChatMember._parse(client, action.participant, users, chats)
-            action = ChatEventAction.MEMBER_INVITED.value
+            action = enums.ChatEventAction.MEMBER_INVITED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionParticipantToggleAdmin):
-            old_admin_rights = types.ChatMember._parse(client, action.prev_participant, users, chats)
-            new_admin_rights = types.ChatMember._parse(client, action.new_participant, users, chats)
-            action = ChatEventAction.ADMIN_RIGHTS_CHANGED.value
+            old_administrator_privileges = types.ChatMember._parse(client, action.prev_participant, users, chats)
+            new_administrator_privileges = types.ChatMember._parse(client, action.new_participant, users, chats)
+            action = enums.ChatEventAction.ADMINISTRATOR_PRIVILEGES_CHANGED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionParticipantToggleBan):
             old_member_permissions = types.ChatMember._parse(client, action.prev_participant, users, chats)
             new_member_permissions = types.ChatMember._parse(client, action.new_participant, users, chats)
-            action = ChatEventAction.MEMBER_PERMISSIONS_CHANGED.value
+            action = enums.ChatEventAction.MEMBER_PERMISSIONS_CHANGED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionStopPoll):
             stopped_poll = await types.Message._parse(client, action.message, users, chats)
-            action = ChatEventAction.POLL_STOPPED.value
+            action = enums.ChatEventAction.POLL_STOPPED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionParticipantJoin):
-            action = ChatEventAction.MEMBER_JOINED.value
+            action = enums.ChatEventAction.MEMBER_JOINED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionParticipantLeave):
-            action = ChatEventAction.MEMBER_LEFT.value
+            action = enums.ChatEventAction.MEMBER_LEFT
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionToggleInvites):
             invites_enabled = action.new_value
-            action = ChatEventAction.INVITES_ENABLED.value
+            action = enums.ChatEventAction.INVITES_ENABLED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionTogglePreHistoryHidden):
             history_hidden = action.new_value
-            action = ChatEventAction.HISTORY_HIDDEN.value
+            action = enums.ChatEventAction.HISTORY_HIDDEN
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionToggleSignatures):
             signatures_enabled = action.new_value
-            action = ChatEventAction.SIGNATURES_ENABLED.value
+            action = enums.ChatEventAction.SIGNATURES_ENABLED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionToggleSlowMode):
             old_slow_mode = action.prev_value
             new_slow_mode = action.new_value
-            action = ChatEventAction.SLOW_MODE_CHANGED.value
+            action = enums.ChatEventAction.SLOW_MODE_CHANGED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionUpdatePinned):
             message = action.message
 
             if message.pinned:
                 pinned_message = await types.Message._parse(client, message, users, chats)
-                action = ChatEventAction.MESSAGE_PINNED.value
+                action = enums.ChatEventAction.MESSAGE_PINNED
             else:
                 unpinned_message = await types.Message._parse(client, message, users, chats)
-                action = ChatEventAction.MESSAGE_UNPINNED.value
+                action = enums.ChatEventAction.MESSAGE_UNPINNED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionExportedInviteEdit):
             old_invite_link = types.ChatInviteLink._parse(client, action.prev_invite, users)
             new_invite_link = types.ChatInviteLink._parse(client, action.new_invite, users)
-            action = ChatEventAction.INVITE_LINK_EDITED.value
+            action = enums.ChatEventAction.INVITE_LINK_EDITED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionExportedInviteRevoke):
             revoked_invite_link = types.ChatInviteLink._parse(client, action.invite, users)
-            action = ChatEventAction.INVITE_LINK_REVOKED
+            action = enums.ChatEventAction.INVITE_LINK_REVOKED
 
         elif isinstance(action, raw.types.ChannelAdminLogEventActionExportedInviteDelete):
             deleted_invite_link = types.ChatInviteLink._parse(client, action.invite, users)
-            action = ChatEventAction.INVITE_LINK_DELETED.value
+            action = enums.ChatEventAction.INVITE_LINK_DELETED
 
         else:
-            action = f"{ChatEventAction.UNKNOWN.value}-{action.QUALNAME}"
+            action = f"{enums.ChatEventAction.UNKNOWN}-{action.QUALNAME}"
 
         return ChatEvent(
             id=event.id,
-            date=event.date,
+            date=utils.timestamp_to_datetime(event.date),
             user=user,
             action=action,
             old_description=old_description,
@@ -574,8 +462,8 @@ class ChatEvent(Object):
 
             invited_member=invited_member,
 
-            old_admin_rights=old_admin_rights,
-            new_admin_rights=new_admin_rights,
+            old_administrator_privileges=old_administrator_privileges,
+            new_administrator_privileges=new_administrator_privileges,
 
             old_member_permissions=old_member_permissions,
             new_member_permissions=new_member_permissions,
